/*
 * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
 *
 * Copyright (C) 1990-1999, William Chia-Wei Cheng.
 *
 * Permission limited to the use, copy, display, distribute without
 * charging for a fee, and produce derivative works of "tgif" and
 * its documentation for not-for-profit purpose is hereby granted by
 * the Author, provided that the above copyright notice appears in
 * all copies made of "tgif" and that both the copyright notice
 * and this permission notice appear in supporting documentation,
 * and that the name of the Author not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.  The Author makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.  All other rights (including, but not limited to, the
 * right to sell "tgif", the right to sell or distribute derivative
 * works of "tgif", the right to distribute "tgif" for a fee, and
 * the right to include "tgif" or derivative works of "tgif" in a
 * for-sale product) are reserved by the Author.
 *
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT
 * OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.50 1999/06/10 11:26:36 william Exp $
 */

#define _INCLUDE_FROM_MENUINFO_C_

#include "tgifdefs.h"
#include "cmdids.h"

#include "align.e"
#include "color.e"
#include "edit.e"
#include "file.e"
#include "font.e"
#include "grid.e"
#include "help.e"
#include "import.e"
#include "menu.e"
#include "menuinfo.e"
#include "names.e"
#include "navigate.e"
#include "page.e"
#include "pattern.e"
#include "raster.e"
#include "scroll.e"
#include "shape.e"
#include "stream.e"
#include "tangram2.e"
#include "text.e"
#include "xbitmap.e"

/* ------------------ color menus ------------------ */

TgMenuInfo colorMenuInfo={ TGMUTYPE_COLOR, NULL, CreateColorMenu };

/* ------------------ bitmap menus ------------------ */

static TgMenuItemInfo modeMenuItemInfo[] = {
   { (char*)(&choicePixmap[NOTHING]), NULL,
     "Enter select/move/resize object mode", NULL, CMDID_SELECTMODE },
   { (char*)(&choicePixmap[DRAWTEXT]), NULL,
     "Enter draw text mode", NULL, CMDID_DRAWTEXT },
   { (char*)(&choicePixmap[DRAWBOX]), NULL,
     "Enter draw rectangle mode", NULL, CMDID_DRAWBOX },
   { (char*)(&choicePixmap[DRAWCORNEROVAL]), NULL,
     "Enter draw corner oval mode", NULL, CMDID_DRAWCORNEROVAL },
   { (char*)(&choicePixmap[DRAWCENTEROVAL]), NULL,
     "Enter draw center oval mode", NULL, CMDID_DRAWCENTEROVAL },
   { (char*)(&choicePixmap[DRAWEDGECIRCLE]), NULL,
     "Enter draw edge circle mode", NULL, CMDID_DRAWEDGEOVAL },
   { (char*)(&choicePixmap[DRAWPOLY]), NULL,
     "Enter draw poly/open spline mode", NULL, CMDID_DRAWPOLY },
   { (char*)(&choicePixmap[DRAWPOLYGON]), NULL,
     "Enter draw polygon/close spline mode", NULL, CMDID_DRAWPOLYGON },
   { (char*)(&choicePixmap[DRAWARC]), NULL,
     "Enter draw arc mode (specify center first)", NULL, CMDID_DRAWARC },
   { (char*)(&choicePixmap[DRAWEDGEARC]), NULL,
     "Enter draw arc mode (specify endpoints first)", NULL, CMDID_DRAWEDGEARC },
   { (char*)(&choicePixmap[DRAWRCBOX]), NULL,
     "Enter draw rcbox mode", NULL, CMDID_DRAWRCBOX },
   { (char*)(&choicePixmap[FREEHAND]), NULL,
     "Enter draw freehand poly/open spline mode", NULL, CMDID_FREEHANDMODE },
   { (char*)(&choicePixmap[VERTEXMODE]), NULL,
     "Enter select/move vertices mode", NULL, CMDID_SELECTVERTEXMODE },
   { (char*)(&choicePixmap[ROTATEMODE]), NULL,
     "Enter rotate/shear object mode", NULL, CMDID_ROTATESHEARMODE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo modeMenuInfo={ TGMUTYPE_BITMAP, modeMenuItemInfo, CreateModeMenu };

TgMenuInfo fillMenuInfo={ TGMUTYPE_BITMAP, NULL, CreateFillMenu };
TgMenuInfo penMenuInfo={ TGMUTYPE_BITMAP, NULL, CreatePenMenu };
TgMenuInfo lineWidthMenuInfo={ TGMUTYPE_BITMAP, NULL, CreateLineWidthMenu };

static TgMenuItemInfo lineStyleMenuItemInfo[] = {
   { (char*)(&lineStylePixmap[LS_PLAIN]), NULL,
     "No arrows for a poly/open spline", NULL, CMDID_CHANGEALLSELLINESTYLE },
   { (char*)(&lineStylePixmap[LS_RIGHT]), NULL,
     "An arrow at the end of a poly/open spline",
     NULL, CMDID_CHANGEALLSELLINESTYLE },
   { (char*)(&lineStylePixmap[LS_LEFT]), NULL,
     "An arrow at the beginning of a poly/open spline",
     NULL, CMDID_CHANGEALLSELLINESTYLE },
   { (char*)(&lineStylePixmap[LS_DOUBLE]), NULL,
     "Arrows at both ends of a poly/open spline",
     NULL, CMDID_CHANGEALLSELLINESTYLE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo lineStyleMenuInfo={ TGMUTYPE_BITMAP, lineStyleMenuItemInfo,
                               CreateLineStyleMenu };

static TgMenuItemInfo lineTypeMenuItemInfo[] = {
   { (char*)(&lineTypePixmap[LT_STRAIGHT]), NULL,
     "Straight line segments", NULL, CMDID_CHANGEALLSELLINETYPE },
   { (char*)(&lineTypePixmap[LT_SPLINE]), NULL,
     "Open/closed splines", NULL, CMDID_CHANGEALLSELLINETYPE },
   { (char*)(&lineTypePixmap[LT_INTSPLINE]), NULL,
     "Interpolated splines", NULL, CMDID_CHANGEALLSELLINETYPE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo lineTypeMenuInfo={ TGMUTYPE_BITMAP, lineTypeMenuItemInfo,
                              CreateLineTypeMenu };

TgMenuInfo lineDashMenuInfo={ TGMUTYPE_BITMAP, NULL, CreateLineDashMenu };

static TgMenuItemInfo horiAlignMenuItemInfo[] = {
   { (char*)(&alignHoriPixmap[ALIGN_N]), NULL,
     "Set horizontal alignment to none",
     NULL, CMDID_SETHORIZONTALALIGN },
   { (char*)(&alignHoriPixmap[ALIGN_L]), NULL,
     "Set horizontal alignment to left aligned",
     NULL, CMDID_SETHORIZONTALALIGN },
   { (char*)(&alignHoriPixmap[ALIGN_C]), NULL,
     "Set horizontal alignment to center aligned",
     NULL, CMDID_SETHORIZONTALALIGN },
   { (char*)(&alignHoriPixmap[ALIGN_R]), NULL,
     "Set horizontal alignment to right aligned",
     NULL, CMDID_SETHORIZONTALALIGN },
   { (char*)(&alignHoriPixmap[ALIGN_S]), NULL,
     "Set horizontal distribution to equal-spacing",
     NULL, CMDID_SETHORIZONTALALIGN },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo horiAlignMenuInfo={ TGMUTYPE_BITMAP, horiAlignMenuItemInfo,
                               CreateHoriAlignMenu };

static TgMenuItemInfo vertAlignMenuItemInfo[] = {
   { (char*)(&alignVertPixmap[ALIGN_N]), NULL,
     "Set vertical alignment to none",
     NULL, CMDID_SETVERTICALALIGN },
   { (char*)(&alignVertPixmap[ALIGN_T]), NULL,
     "Set vertical alignment to top aligned",
     NULL, CMDID_SETVERTICALALIGN },
   { (char*)(&alignVertPixmap[ALIGN_M]), NULL,
     "Set vertical alignment to middle aligned",
     NULL, CMDID_SETVERTICALALIGN },
   { (char*)(&alignVertPixmap[ALIGN_B]), NULL,
     "Set vertical alignment to bottom aligned",
     NULL, CMDID_SETVERTICALALIGN },
   { (char*)(&alignVertPixmap[ALIGN_S]), NULL,
     "Set vertical distribution to equal-spacing",
     NULL, CMDID_SETVERTICALALIGN },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo vertAlignMenuInfo={ TGMUTYPE_BITMAP, vertAlignMenuItemInfo,
                               CreateVertAlignMenu };

static TgMenuItemInfo moveModeMenuItemInfo[] = {
   { (char*)(&moveModePixmap[CONST_MOVE]), NULL,
     "Set constrained move mode", NULL, CMDID_SETMOVEMODE },
   { (char*)(&moveModePixmap[UNCONST_MOVE]), NULL,
     "Set unconstrained move mode", NULL, CMDID_SETMOVEMODE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo moveModeMenuInfo={ TGMUTYPE_BITMAP, moveModeMenuItemInfo,
                              CreateMoveModeMenu };

static TgMenuItemInfo stretchableTextModeMenuItemInfo[] = {
   { (char*)(&stretchableModePixmap[NO_STRETCHABLE_TEXT]), NULL,
     "Make text objects stretchable", NULL, CMDID_SETSTRETCHTEXTMODE },
   { (char*)(&stretchableModePixmap[STRETCHABLE_TEXT]), NULL,
     "Make text objects not stretchable", NULL, CMDID_SETSTRETCHTEXTMODE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo stretchableTextModeMenuInfo={ TGMUTYPE_BITMAP,
                                         stretchableTextModeMenuItemInfo,
                                         CreateStretchableTextModeMenu };

TgMenuInfo shapeMenuInfo={ TGMUTYPE_BITMAP, NULL, CreateShapeMenu };

static TgMenuItemInfo pageLayoutMenuItemInfo[] = {
   { (char*)(&pageLayoutPixmap[PAGE_STACK]), NULL,
     "Stacked page mode", NULL, CMDID_SETPAGELAYOUTMODE },
   { (char*)(&pageLayoutPixmap[PAGE_TILE]), NULL,
     "Tiled page mode", NULL, CMDID_SETPAGELAYOUTMODE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo pageLayoutMenuInfo={ TGMUTYPE_BITMAP, pageLayoutMenuItemInfo,
                                CreatePageLayoutMenu };

static TgMenuItemInfo transPatModeMenuItemInfo[] = {
   { (char*)(&transPatPixmap[NO_TRANSPAT_MODE]), NULL,
     "Make fill and pen patterns opaque", NULL, CMDID_SETTRANSPATMODE },
   { (char*)(&transPatPixmap[TRANSPAT_MODE]), NULL,
     "Make fill and pen patterns transparent", NULL, CMDID_SETTRANSPATMODE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo transPatModeMenuInfo={ TGMUTYPE_BITMAP, transPatModeMenuItemInfo,
                                  CreateTransPatModeMenu };

static TgMenuItemInfo pageStyleMenuItemInfo[] = {
   { N_("Portrait"), NULL, "Portrait page style", NULL, CMDID_PORTRAITMODE },
   { N_("Landscape"), NULL, "Lanscape page style", NULL,
     CMDID_LANDSCAPEMODE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo pageStyleMenuInfo={ TGMUTYPE_TEXT, pageStyleMenuItemInfo,
                               CreatePageStyleMenu };

/* ------------------ text menus ------------------ */

TgMenuInfo fontMenuInfo={ TGMUTYPE_TEXT, NULL, CreateFontMenu };

static TgMenuItemInfo fontStyleMenuItemInfo[] = {
   { N_("Roman"), "Alt+Cntrl+o", "Roman text style", NULL,
     CMDID_CHANGEALLSELFONTSTYLE },
   { N_("Bold"), "Alt+Cntrl+b", "Bold text style", NULL,
     CMDID_CHANGEALLSELFONTSTYLE },
   { N_("Italic"), "Alt+Cntrl+t", "Italic text style", NULL,
     CMDID_CHANGEALLSELFONTSTYLE },
   { N_("BoldItalic"), "Alt+Cntrl+p", "Bold-italic text style", NULL,
     CMDID_CHANGEALLSELFONTSTYLE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Left"), "Alt+Cntrl+l", "Left justified text", NULL,
     CMDID_CHANGEALLSELFONTJUST },
   { N_("Center"), "Alt+Cntrl+c", "Center justified text", NULL,
     CMDID_CHANGEALLSELFONTJUST },
   { N_("Right"), "Alt+Cntrl+r", "Right justified text", NULL,
     CMDID_CHANGEALLSELFONTJUST },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("NoUnderlined"), NULL, "No underlined text", NULL,
     CMDID_CHANGEALLSELFONTUNDERLINE },
   { N_("Underlined"), NULL, "Underlined text", NULL,
     CMDID_CHANGEALLSELFONTUNDERLINE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo fontStyleMenuInfo={ TGMUTYPE_TEXT, fontStyleMenuItemInfo,
                               CreateFontStyleMenu };

TgMenuInfo fontSizeMenuInfo={ TGMUTYPE_TEXT, NULL, CreateFontSizeMenu };

static TgMenuItemInfo stackedPageMenuItemInfo[] = {
   { N_("NextPage"), NULL, "Go to next page", NULL, CMDID_NEXTPAGE },
   { N_("PrevPage"), NULL, "Go to previous page", NULL, CMDID_PREVPAGE },
   { N_("GotoPage..."), NULL, "Go to a specified page", NULL, CMDID_GOTOPAGE },
   { N_("NamePages..."), NULL, "Assign names to pages", NULL, CMDID_NAMEPAGES },
   { N_("EditPageFileNames..."), NULL,
     "Assign file names to pages to be used for PrintOneFilePerPage()", NULL,
     CMDID_EDITPAGEFILENAMES },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("AddPageBefore..."), NULL, "Add a page before the current page", NULL,
     CMDID_ADDPAGEBEFORE },
   { N_("AddPageAfter..."), NULL, "Add a page after the current page", NULL,
     CMDID_ADDPAGEAFTER },
   { N_("DeleteCurPage"), NULL, "Delete the current page", NULL,
     CMDID_DELETECURPAGE },
   { N_("DeletePages..."), NULL, "Delete user-specified pages", NULL,
     CMDID_DELETEPAGES },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("PrintOnePage"), NULL, "Print the curret page", NULL,
     CMDID_PRINTONEPAGE },
   { N_("PrintOneFilePerPage"), NULL,
     "Print into a separate file for each page",
     NULL, CMDID_PRINTONEFILEPERPAGE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SpecifyPaperSize..."), NULL, "Set the physical size of the paper",
     NULL, CMDID_SPECIFYPAPERSIZE },
   { N_("SetPaperColor..."), NULL, "Set the color of the paper", NULL,
     CMDID_SETPAPERCOLOR },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo stackedPageMenuInfo={ TGMUTYPE_TEXT, stackedPageMenuItemInfo,
                                 TgCreateMenuFromMenuInfo };

static TgMenuItemInfo tiledPageMenuItemInfo[] = {
   { N_("TogglePageLineShown"), NULL, "Toggle the showing of page boundaries",
     NULL, CMDID_TOGGLEPAGELINESHOWN },
   { N_("SpecifyDrawingSize..."), NULL, "Change the drawing size", NULL,
     CMDID_SPECIFYDRAWINGSIZE },
   { N_("PrintOnePage"), NULL, "Select and print one of the pages", NULL,
     CMDID_PRINTONEPAGE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SpecifyPaperSize..."), NULL, "Set the physical size of the paper",
     NULL, CMDID_SPECIFYPAPERSIZE },
   { N_("SetPaperColor..."), NULL, "Set the color of the paper", NULL,
     CMDID_SETPAPERCOLOR },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo tiledPageMenuInfo={ TGMUTYPE_TEXT, tiledPageMenuItemInfo,
                               TgCreateMenuFromMenuInfo };

static TgMenuItemInfo pageMenuItemInfo[] = {
   { N_("StackedPage"), TGMUITEM_PINNABLESUBMENU,
     "StackedPage Menu", &stackedPageMenuInfo, MENU_STACKEDPAGE },
   { N_("TiledPage"), TGMUITEM_PINNABLESUBMENU,
     "TiledPage Menu", &tiledPageMenuInfo, MENU_TILEDPAGE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo pageMenuInfo={ TGMUTYPE_TEXT, pageMenuItemInfo, CreatePageMenu };

static TgMenuItemInfo reduceColorMenuItemInfo[] = {
   { N_("ReduceColors..."), NULL,
     "Reduce the number of colors of a selected image",
     NULL, CMDID_REDUCECOLORS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetDefaultColorLevels..."), NULL,
     "Set the number of R G B color levels for the ``standard colors''", NULL,
     CMDID_SETDEFAULTCOLORLEVELS },
   { N_("ReduceToPixmapColors..."), NULL,
     "Reduce the colors of a selected image to a the colors in an XPM file",
     NULL, CMDID_REDUCETOPIXMAPCOLORS },
   { N_("ReduceToDefaultColors"), NULL,
     "Reduce the colors of a selected image to standard colors", NULL,
     CMDID_REDUCETODEFAULTCOLORs },
   { N_("DefaultErrorDiffuse"), NULL,
     "Error diffuse to reduce colors of a selected image to standard colors",
     NULL, CMDID_DEFAULTERRORDIFFUSE },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo reduceColorMenuInfo={ TGMUTYPE_TEXT, reduceColorMenuItemInfo,
                                        TgCreateMenuFromMenuInfo };

static TgMenuItemInfo effectsMenuItemInfo[] = {
   { N_("EdgeDetect"), NULL, "Perform edge-detection on a selected image", NULL,
     CMDID_EDGEDETECT },
   { N_("Emboss"), NULL, "Convert a selected image to gray and then emboss",
     NULL, CMDID_EMBOSS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Spread..."), NULL, "Spread the pixels of a selected image around",
     NULL, CMDID_SPREAD },
   { N_("Sharpen"), NULL, "Sharpen a selected image", NULL, CMDID_SHARPEN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Blur(3x3)"), NULL, "Blur a selected image using a 3 by 3 filter", NULL,
     CMDID_BLUR3 },
   { N_("Blur(5x5)"), NULL, "Blur a selected image using a 5 by 5 filter", NULL,
     CMDID_BLUR5 },
   { N_("Blur(7x7)"), NULL, "Blur a selected image using a 7 by 7 filter", NULL,
     CMDID_BLUR7 },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo effectsMenuInfo={ TGMUTYPE_TEXT, effectsMenuItemInfo,
                                    TgCreateMenuFromMenuInfo };

static TgMenuItemInfo generateMenuItemInfo[] = {
   { N_("RunBggen..."), NULL,
     "Run 'bggen' to create an image using the size of a selected image", NULL,
     CMDID_RUNBGGEN },
   { N_("CircularBggen..."), NULL,
     "Create a gray circular image using the size of a selected image", NULL,
     CMDID_CIRCULARBGGEN },
   { N_("SimpleRectBggen..."), NULL,
     "Create a single-color rectanglar image using the size of selected image",
     NULL, CMDID_SIMPLERECTBGGEN },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo generateMenuInfo={ TGMUTYPE_TEXT, generateMenuItemInfo,
                                     TgCreateMenuFromMenuInfo };

static TgMenuItemInfo imageProcMenuItemInfo[] = {
   { N_("MakeGray"), NULL, "Convert a selected image into gray-scale", NULL,
     CMDID_MAKEGRAY },
   { N_("Brighten/Darken..."), NULL, "Brighten or darken a selected image",
     NULL, CMDID_BRIGHTENDARKEN },
   { N_("ContrastEnhance..."), NULL, "Adjust contrast of a selected image",
     NULL, CMDID_CONTRASTENHANCE },
   { N_("GammaCorrect..."), NULL, "Apply gamma correction a selected image",
     NULL, CMDID_GAMMA },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ChangeSaturation..."), NULL, "Change saturation of a selected image",
     NULL, CMDID_CHANGESATURATION },
   { N_("ChangeHue..."), NULL, "Change hue of a selected image", NULL,
     CMDID_CHANGEHUE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("InvertColor"), NULL,
     "Subtract every pixel of a selected image from White (in RGB-model)",
     NULL, CMDID_INVERTCOLOR },
   { N_("InterpolateColor..."), NULL,
     "Map pixel intensities of a selected image between two specified colors",
     NULL, CMDID_INTERPOLATECOLOR },
   { N_("ColorBalance..."), NULL,
     "Balance the RGB components of a selected image",
     NULL, CMDID_COLORBALANCE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ReduceNumberOfColors"), TGMUITEM_SUBMENU,
     "Reduce number of colors submenu >>>", &reduceColorMenuInfo, INVALID },
   { N_("Effects"), TGMUITEM_SUBMENU,
     "Effects submenu >>>", &effectsMenuInfo, INVALID },
   { N_("Generate"), TGMUITEM_SUBMENU,
     "Generate submenu >>>", &generateMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("RegenerateImage"), NULL,
     "Regenerated a selected image", NULL, CMDID_REGENERATEIMAGE },
   { N_("CropImage"), NULL, "Crop a selected image", NULL, CMDID_CROPIMAGE },
   { N_("GetColor"), NULL,
     "Pick a color from a selected image to be the current color", NULL,
     CMDID_GETCOLOR },
   { N_("ReplaceColor"), NULL,
     "Replace a color in a selected image with the current color", NULL,
     CMDID_REPLACECOLOR },
   { N_("FloodFill"), NULL,
     "Flood-filling a selected image with the current color",
     NULL, CMDID_FLOODFILL },
   { N_("CreateContour"), NULL,
     "Generate a contour from a point in a selected image", NULL,
     CMDID_CREATECONTOUR },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Subtract"), NULL,
     "Subtract one image from another", NULL, CMDID_SUBTRACT },
   { N_("AlphaCombine"), NULL,
     "Combine 2 images using another image as the alpha channel", NULL,
     CMDID_ALPHACOMBINE },
   { N_("XorColor"), NULL, "XOR the colors of one image into another", NULL,
     CMDID_XORCOLORS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("VectorWarp"), NULL,
     "Warp selected image in a specified direction for a specified amount",
     NULL, CMDID_VECTORWARP },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo imageProcMenuInfo={ TGMUTYPE_TEXT, imageProcMenuItemInfo,
                               TgCreateMenuFromMenuInfo };

static TgMenuItemInfo driverMenuItemInfo[] = {
   { N_("Solve"), "Alt+s", "Escape to driver", NULL, CMDID_SOLVE },
   { N_("Simulate"), "Alt+y", "Escape to driver", NULL, CMDID_SIMULATE },
   { N_("Probe"), "Alt+b", "Escape to driver", NULL, CMDID_PROBE },
   { N_("Animate"), "Cntrl+z", "Escape to driver", NULL, CMDID_ANIMATE },
   { N_("Escape"), "Alt+x", "Escape to driver", NULL, CMDID_ESCAPE },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo driverMenuInfo={ TGMUTYPE_TEXT, driverMenuItemInfo,
                                   TgCreateMenuFromMenuInfo };

static TgMenuItemInfo importMenuItemInfo[] = {
   { N_("Import..."), "Alt+p", "Embed/Import another drawing", NULL,
     CMDID_IMPORT },
   { N_("ImportXBitmap..."), "Alt+Cntrl+.", "Embed/Import an X11 bitmap file",
     NULL, CMDID_IMPORTXBITMAP },
   { N_("ImportXPixmap..."), "Alt+Cntrl+,", "Embed/Import an X11 pixmap file",
     NULL, CMDID_IMPORTXPIXMAP },
   { N_("ImportEPSFile..."), "Alt+(", "Link to an PS/EPS file", NULL,
     CMDID_IMPORTEPSFILE },
   { N_("ImportGIFFile..."), NULL, "Embed/Import a GIF file", NULL,
     CMDID_IMPORTGIFFILE },
   { N_("ImportOtherFile..."), NULL, "Embed/Import another type of file", NULL,
     CMDID_IMPORTOTHERFILE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("EmbedEPSFile..."), NULL, "Embed/Import an PS/EPS file", NULL,
     CMDID_EMBEDEPSFILE },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo importMenuInfo={ TGMUTYPE_TEXT, importMenuItemInfo,
                                   TgCreateMenuFromMenuInfo };

static TgMenuItemInfo browseMenuItemInfo[] = {
   { N_("BrowseXBitmap"), NULL, "Recursively read in all X11 bitmap files",
     NULL, CMDID_BROWSEXBITMAP },
   { N_("BrowseXPixmap"), NULL, "Recursively read in all X11 pixmap files",
     NULL, CMDID_BROWSEXPIXMAP },
   { N_("BrowseOther..."), NULL,
     "Recursively read in all another type of files", NULL, CMDID_BROWSEOTHER },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo browseMenuInfo={ TGMUTYPE_TEXT, browseMenuItemInfo,
                                   TgCreateMenuFromMenuInfo };

static TgMenuItemInfo domainMenuItemInfo[] = {
   { N_("ChangeDomain..."), "Cntrl+y", "Change to a different domain", NULL,
     CMDID_CHANGEDOMAIN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Edit Domain Paths..."), NULL, "Edit domain paths", NULL,
     CMDID_EDITDOMAINPATHS },
   { N_("Select Default Domain..."), NULL, "Select the default domain", NULL,
     CMDID_SELECTDEFAULTDOMAIN },
   { N_("Add A Domain..."), NULL, "Add a new domain", NULL,
     CMDID_ADDADOMAIN },
   { N_("Delete A Domain..."), NULL, "Delete an existing domain", NULL,
     CMDID_DELETEADOMAIN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Reload Domain Info From X..."), NULL,
     "Reload domain information from X defaults", NULL,
     CMDID_RELOADDOMAININFOFROMX },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo domainMenuInfo={ TGMUTYPE_TEXT, domainMenuItemInfo,
                                   CreateDomainMenu };

TgMenuInfo exportFormatMenuInfo={ TGMUTYPE_BITMAP, NULL,
                                  CreateExportFormatMenu };

static TgMenuItemInfo gifAnimateMenuItemInfo[] = {
   { N_("ExportXPixmapDeckToGIF"), NULL,
     "Export a deck of X11 pixmap images to an animated GIF file", NULL,
     CMDID_EXPORTXPMDECKTOGIF },
   { N_("ImportGIFToXPixmapDeck..."), NULL,
     "Import an animated GIF file into a deck of X11 pixmap images", NULL,
     CMDID_IMPORTGIFTOXPMDECK },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo gifAnimateMenuInfo={ TGMUTYPE_TEXT, gifAnimateMenuItemInfo,
                                       TgCreateMenuFromMenuInfo };

static TgMenuItemInfo captureMenuItemInfo[] = {
   { N_("Screen Capture"), NULL, "Capture a rectangular area of the screen",
     NULL, CMDID_SCREENCAPTURE },
   { N_("Full Screen Capture"), NULL, "Capture the whole screen", NULL,
     CMDID_FULLSCREENCAPTURE },
   { N_("Delayed Full Screen Capture..."), NULL,
     "Capture the whole screen after a specified delay", NULL,
     CMDID_DELAYEDFULLSCREENCAPTURE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Hide During Capture"), NULL,
     "Toggle between hiding and showing tgif during capture", NULL,
     CMDID_TOGGLEHIDEDURINGCAPTURE },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo captureMenuInfo={ TGMUTYPE_TEXT, captureMenuItemInfo,
                                    CreateScreenCaptureMenu };

#ifdef _TGIF_DBG
static TgMenuItemInfo mimeMenuItemInfo[] = {
   { N_("FakeUserAgent..."), NULL,
     "Set 'UserAgent' to be used when making an HTTP request", NULL,
     CMDID_FAKEUSERAGENT },
   { N_("FakeReferer..."), NULL,
     "Set 'Referer' to be used when making an HTTP request", NULL,
     CMDID_FAKEREFERER },
   { N_("ToggleKeepAlive"), NULL,
     "Toggle between using Keep-Alive and non-Keep-Alive HTTP connections",
     NULL, CMDID_TOGGLEKEEPALIVE },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo mimeMenuInfo={ TGMUTYPE_TEXT, mimeMenuItemInfo,
                                 TgCreateMenuFromMenuInfo };
#endif /* _TGIF_DBG */

static TgMenuItemInfo fileMenuItemInfo[] = {
   { N_("New"), "Cntrl+n", "Start with a blank/new drawing", NULL, CMDID_NEW },
   { N_("Open..."), "Cntrl+o", "Open an existing drawing", NULL, CMDID_OPEN },
   { N_("Import"), TGMUITEM_SUBMENU,
     "Import submenu >>>", &importMenuInfo, INVALID },
   { N_("Browse"), TGMUITEM_SUBMENU,
     "Browse submenu >>>", &browseMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Save"), "Cntrl+s", "Save current drawing", NULL, CMDID_SAVE },
   { N_("SaveNew..."), "Alt+Cntrl+s",
     "Save current drawing in a different file", NULL, CMDID_SAVENEW },
   { N_("SaveSelectedAs..."), "Alt+~",
     "Save selected objects in a different file", NULL, CMDID_SAVESELECTEDAS },
   { N_("SaveSymInLibrary..."), NULL,
     "Save symbol file in a library path within the current domain", NULL,
     CMDID_SAVESYMINLIBRARY },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Domain"), TGMUITEM_SUBMENU,
     "Domain submenu >>>", &domainMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Print"), "Cntrl+p", "Print/export current drawing", NULL,
     CMDID_PRINT },
   { N_("Print/ExportFormat"), TGMUITEM_SUBMENU,
     "Select print/export format submenu >>>", &exportFormatMenuInfo, INVALID },
   { N_("Print/ExportInColor"), "Alt+Cntrl+k",
     "Toggle between color and black & white printing modes", NULL,
     CMDID_TOGGLEBWCOLORPS },
   { N_("PrintWithCmd..."), "Cntrl+-",
     "Print/export current drawing with a specific command", NULL,
     CMDID_PRINTWITHCMD },
   { N_("PrintSelectedObjs"), NULL, "Print/export only selected objects", NULL,
     CMDID_PRINTSELECTEDOBJS },
   { N_("PrintOnePage"), NULL, "Print a page in a multipage drawing", NULL,
     CMDID_PRINTONEPAGE },
   { N_("PrintOneFilePerPage"), NULL,
     "Print into a separate file for each page",
     NULL, CMDID_PRINTONEFILEPERPAGE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("GIFAnimation"), TGMUITEM_SUBMENU,
     "GIFAnimation submenu >>>", &gifAnimateMenuInfo, INVALID },
   { N_("Screen Capture"), TGMUITEM_SUBMENU,
     "Screen Capture submenu >>>", &captureMenuInfo, INVALID },
#ifdef _TGIF_DBG
   { N_("Mime"), TGMUITEM_SUBMENU, "Mime submenu >>>",
     &mimeMenuInfo, INVALID },
#endif /* _TGIF_DBG */
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("PrintMsgBuffer..."), NULL,
     "Display/save the contents of the message window", NULL,
     CMDID_PRINTMSGBUFFER },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetExportPixelTrim..."), NULL,
     "Specify the number of pixels to trim when exporting", NULL,
     CMDID_SETEXPORTPIXELTRIM },
   { N_("SetTemplate..."), NULL,
     "Select a template file be used in STACKED page mode", NULL,
     CMDID_SETTEMPLATE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Return"), TGMUITEM_SUBMENU,
     N_("Return submenu >>>"), &driverMenuInfo, INVALID },
   { N_("Quit"), "Cntrl+q", "Exit <<PROGRAM_NAME>>", NULL, CMDID_QUIT },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo fileMenuInfo={ TGMUTYPE_TEXT, fileMenuItemInfo, CreateFileMenu };

static TgMenuItemInfo transformMenuItemInfo[] = {
   { N_("PreciseScale..."), "Alt+)", "Scale selected objects", NULL,
     CMDID_PRECISESCALE },
   { "PreciseScaleEverything...", NULL, "Scale everything", NULL,
     CMDID_PRECISESCALEEVERYTHING },
   { N_("PreciseRotate..."), NULL,
     "Rotate selected objects by a specified number of degrees", NULL,
     CMDID_PRECISEROTATE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("RestoreImageWH"), NULL,
     "Restore a seleted bitmap/pixmap/EPS object to its original dimension",
     NULL, CMDID_RESTOREIMAGEWH },
   { N_("NoTransform"), NULL,
     "Get rid of transformations on selected simple objects", NULL,
     CMDID_NOTRANSFORM },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo transformMenuInfo={ TGMUTYPE_TEXT, transformMenuItemInfo,
                                      TgCreateMenuFromMenuInfo };

static TgMenuItemInfo bitPixmapMenuItemInfo[] = {
   { N_("InvertXBitmap"), "Alt+Cntrl+f",
     "Invert pixels for selected bitmap objects", NULL, CMDID_INVERTXBITMAP },
   { N_("CutBit/Pixmap..."), "Alt+;",
     "Trim/scale a selected bitmap/pixmap object", NULL, CMDID_CUTBITPIXMAP },
   { N_("BreakUpBit/Pixmap..."), NULL,
     "Break selected bitmap/pixmap object into smaller objects", NULL,
     CMDID_BREAKUPBITPIXMAP },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ExportHalfToneBitmap"), NULL,
     "Use Floyd-Steinberg half-tone method when exporting an X11 bitmap file",
     NULL, CMDID_EXPORTHALFTONEBITMAP },
   { N_("ExportThresholdBitmap"), NULL,
     "Use simple thresholding method when exporting an X11 bitmap file",
     NULL, CMDID_EXPORTTHRESHOLDBITMAP },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetExportBitmapThreshold..."), NULL,
     "Set bitmap threshold value for the simple thresholding method",
     NULL, CMDID_SETEXPORTBITMAPTHRESHOLD },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("UpdateEPS"), NULL, "Refresh a selected EPS object", NULL,
     CMDID_UPDATEEPS },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo bitPixmapMenuInfo={ TGMUTYPE_TEXT, bitPixmapMenuItemInfo,
                                      CreateBitPixmapMenu };

static TgMenuItemInfo polyMenuItemInfo[] = {
   { N_("DeletePoint"), "Alt+Cntrl+d",
     "Delete vertices from a poly/polygon/spline", NULL, CMDID_DELETEPOINT },
   { N_("AddPoint"), "Alt+Cntrl+a", "Add vertices to a polygon/polygon/spline",
     NULL, CMDID_ADDPOINT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ConvertIntSpline"), NULL,
     "Convert selected interpolated splines to regular splines", NULL,
     CMDID_CONVERTINTSPLINE },
   { N_("Smooth<->Hinge"), NULL,
     "Toggle smooth/hinge vertices for a selected poly/polygon/spline", NULL,
     CMDID_TOGGLESMOOTHHINGE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("JoinPoly"), NULL, "Join polylines or open-splines at endpoints", NULL,
     CMDID_JOINPOLY },
   { N_("CutPoly"), NULL,
     "Cut a poly/polygon at a vertex into two pieces", NULL, CMDID_CUTPOLY },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo polyMenuInfo={ TGMUTYPE_TEXT, polyMenuItemInfo,
                                 TgCreateMenuFromMenuInfo };

static TgMenuItemInfo textMenuItemInfo[] = {
   { N_("BreakUpText"), "Alt+#",
     "Break up selected text objects into one-character text objects", NULL,
     CMDID_BREAKUPTEXT },
   { N_("SetTextFillPatternColor"), NULL,
     "Set the fill pattern color of a text object to be the current color",
     NULL, CMDID_SETTEXTFILLPATTERNCOLOR },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("InsertRightSuperscript"), "Alt+Cntrl+6",
     "Insert superscript to the right of the text cursor", NULL,
     CMDID_INSERTRIGHTSUPERSCRIPT },
   { N_("InsertRightSubscript"), NULL,
     "Insert subscript to the right of the text cursor", NULL,
     CMDID_INSERTRIGHTSUBSCRIPT },
   { N_("InsertLeftSuperscript"), NULL,
     "Insert superscript to the left of the text cursor", NULL,
     CMDID_INSERTLEFTSUPERSCRIPT },
   { N_("InsertLeftSubscript"), NULL,
     "Insert subscript to the left of the text cursor", NULL,
     CMDID_INSERTLEFTSUBSCRIPT },
   { N_("InsertCenterSuperscript"), NULL,
     "Insert superscript above the text cursor", NULL,
     CMDID_INSERTCENTERSUPERSCRIPT },
   { N_("InsertCenterSubscript"), NULL,
     "Insert subscript below the text cursor", NULL,
     CMDID_INSERTCENTERSUBSCRIPT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("InsertThinSpace..."), NULL,
     "Insert a think space at the text cursor", NULL,
     CMDID_INSERTTHINSPACE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("InsertVerticalOffset..."), NULL,
     "Insert a vertical offset at the text cursor", NULL,
     CMDID_INSERTVERTICALOFFSET },
   { N_("RemoveVerticalOffset..."), NULL,
     "Remove any vertical offset at the text cursor", NULL,
     CMDID_REMOVEVERTICALOFFSET },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetScriptFraction"), NULL,
     "Set the relative size of super/subscript text", NULL,
     CMDID_SETSCRIPTFRACTION },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo textMenuInfo={ TGMUTYPE_TEXT, textMenuItemInfo,
                                 CreateEditTextMenu };

static TgMenuItemInfo createObjectMenuItemInfo[] = {
   { N_("InputPolyPts"), NULL, "Read points from terminal and create a poly",
     NULL, CMDID_INPUTPOLYPTS },
   { N_("InputPolygonPts"), NULL,
     "Read points from terminal and create a polygon",
     NULL, CMDID_INPUTPOLYGONPTS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SpecifyAnArc..."), "Alt+9", "Create and specify an arc", NULL,
     CMDID_SPECIFYANARC },
   { N_("MakeRegPolygon..."), "Alt+\"",
     "Make a selected polygon/closed spline into a regular one", NULL,
     CMDID_MAKEREGPOLYGON },
   { N_("GetBoundingBox"), NULL,
     "Get rectangular bounding boxes of selected objects", NULL,
     CMDID_GETBOUNDINGBOX },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo createObjectMenuInfo={ TGMUTYPE_TEXT,
                                         createObjectMenuItemInfo,
                                         TgCreateMenuFromMenuInfo };

static TgMenuItemInfo editMenuItemInfo[] = {
   { N_("Redraw"), "Cntrl+r", "Redraw canvas window", NULL, CMDID_REDRAW },
   { N_("Duplicate"), "Cntrl+d", "Duplicate selected objects", NULL,
     CMDID_DUPLICATE },
   { N_("Delete"), "Cntrl+x", "Delete selected objects", NULL, CMDID_DELETE },
   { N_("SelectAll"), "Cntrl+a", "Select all objects on the current page", NULL,
     CMDID_SELECTALL },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Undo"), "Alt+u", "Undo the most recent command", NULL, CMDID_UNDO },
   { N_("Redo"), "Alt+*", "Redo the most undone command", NULL, CMDID_REDO },
   { N_("FlushUndoBuffer"), NULL, "Flush undo buffer and unused colors", NULL,
     CMDID_FLUSHUNDOBUFFER },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Copy"), "Cntrl+c", "Copy selected objects into the cut buffer", NULL,
     CMDID_COPY },
   { N_("CopyPlainTextAsObject"), NULL,
     "Copy highlighted plain text as an object into the cut buffer", NULL,
     CMDID_COPYPLAINTEXTASOBJECT },
   { N_("Cut"), NULL, "Cut selected objects into the cut buffer", NULL,
     CMDID_CUT },
   { N_("Paste"), "Cntrl+v", "Paste from the cut buffer", NULL,
     CMDID_PASTE },
   { N_("PasteFromFile..."), NULL,
     "Select a file and paste the content as text", NULL, CMDID_PASTEFROMFILE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Shape"), TGMUITEM_PINNABLESUBMENU,
     "Shape submenu >>>", &shapeMenuInfo, MENU_SHAPE },
   { N_("SetShapeShadow..."), NULL, "Specify the shadow for shapes", NULL,
     CMDID_SETSHAPESHADOW },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ImageProc"), TGMUITEM_PINNABLESUBMENU,
     "ImageProc submenu >>>", &imageProcMenuInfo, MENU_IMAGEPROC },
   { N_("Properties"), TGMUITEM_PINNABLESUBMENU,
     "Properties submenu >>>", &propertiesMenuInfo, MENU_PROPERTIES },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Transformation"), TGMUITEM_SUBMENU,
     "Transformation submenu >>>", &transformMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Bitmap/Pixmap/EPS"), TGMUITEM_SUBMENU,
     "Bitmap/Pixmap/EPS submenu >>>", &bitPixmapMenuInfo, INVALID },
   { N_("Poly/Polygon"), TGMUITEM_SUBMENU,
     "Poly/Polygon submenu >>>", &polyMenuInfo, INVALID },
   { N_("Text"), TGMUITEM_SUBMENU, "Text submenu >>>", &textMenuInfo, INVALID },
   { N_("CreateObject"), TGMUITEM_SUBMENU,
     "Create object submenu >>>", &createObjectMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("PushCurChoice"), "Cntrl+e", "Save/restore current drawing mode", NULL,
     CMDID_PUSHCURCHOICE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("FindCaseSensitive..."), "Alt+Cntrl+0",
     "Find string in visible text objects (case sensitive)", NULL,
     CMDID_FINDCASESENSITIVE },
   { N_("FindNoCase..."), "Alt+Cntrl+9",
     "Find string in visible text objects (case insensitive)", NULL,
     CMDID_FINDNOCASE },
   { N_("FindAgain"), "Alt+Cntrl+=", "Repeat the last Find command", NULL,
     CMDID_FINDAGAIN },
/*
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { "ConnectPins", NULL, "Connect pins", NULL, CMDID_CONNECTPINS },
 */
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo editMenuInfo={ TGMUTYPE_TEXT, editMenuItemInfo, CreateEditMenu };

static TgMenuItemInfo alignMenuItemInfo[] = {
   { N_("AlignObjsTop"), "Alt+{", "Align top sides of selected objects", NULL,
     CMDID_ALIGNOBJSTOP },
   { N_("AlignObjsMiddle"), "Alt++",
     "Align vertical centers of selected objects",
     NULL, CMDID_ALIGNOBJSMIDDLE },
   { N_("AlignObjsBottom"), "Alt+}",
     "Align bottom sides of selected objects", NULL, CMDID_ALIGNOBJSBOTTOM },
   { N_("AlignObjsLeft"), "Alt+[", "Align left sides of selected objects", NULL,
     CMDID_ALIGNOBJSLEFT },
   { N_("AlignObjsCenter"), "Alt+=",
     "Align horizontal centers of selected objects",
     NULL, CMDID_ALIGNOBJSCENTER },
   { N_("AlignObjsRight"), "Alt+]", "Align right sides of selected objects",
     NULL, CMDID_ALIGNOBJSRIGHT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("CenterAnEndPoint"), NULL,
     "Move an endpoint of a selected poly to the center of another object",
     NULL, CMDID_CENTERANENDPOINT },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo alignMenuInfo={ TGMUTYPE_TEXT, alignMenuItemInfo,
                                  TgCreateMenuFromMenuInfo };

static TgMenuItemInfo sizeMenuItemInfo[] = {
   { N_("SizeToWidest"), NULL,
     "Resize all selected objects to be as wide as the widest one", NULL,
     CMDID_SIZETOWIDEST },
   { N_("SizeToNarrowest"), NULL,
     "Resize all selected objects to be as narrow as the narrowest one", NULL,
     CMDID_SIZETONARROWEST },
   { N_("SizeToTallest"), NULL,
     "Resize all selected objects to be as tall as the tallest one", NULL,
     CMDID_SIZETOTALLEST },
   { N_("SizeToShortest"), NULL,
     "Resize all selected objects to be as short as the shortest one", NULL,
     CMDID_SIZETOSHORTEST },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SizeToGivenWidthHeight..."), NULL,
     "Resize all selected objects to a specified width and height", NULL,
     CMDID_SIZETOGIVENWIDTHHEIGHT },
   { N_("SizeToGivenWidth..."), NULL,
     "Resize all selected objects to a specified width", NULL,
     CMDID_SIZETOGIVENWIDTH },
   { N_("SizeToGivenHeight..."), NULL,
     "Resize all selected objects to a specified height", NULL,
     CMDID_SIZETOGIVENHEIGHT },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo sizeMenuInfo={ TGMUTYPE_TEXT, sizeMenuItemInfo,
                                 TgCreateMenuFromMenuInfo };

static TgMenuInfo alignObjsDirectMenuInfo={ TGMUTYPE_BITMAP, NULL,
                                            CreateAlignObjsDirectMenu };
static TgMenuInfo alignToGridDirectMenuInfo={ TGMUTYPE_BITMAP, NULL,
                                              CreateAlignToGridDirectMenu };
static TgMenuInfo alignToPageDirectMenuInfo={ TGMUTYPE_BITMAP, NULL,
                                              CreateAlignToPageDirectMenu };

static TgMenuInfo distrObjsDirectMenuInfo={ TGMUTYPE_BITMAP, NULL,
                                            CreateDistributeDirectMenu };

static TgMenuItemInfo flipRotateMenuItemInfo[] = {
   { N_("FlipHorizontal"), "Alt+h", "Flip selected objects horizontally", NULL,
     CMDID_FLIPHORIZONTAL },
   { N_("FlipVertical"), "Alt+v", "Flip selected objects vertically", NULL,
     CMDID_FLIPVERTICAL },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("RotateClockWise"), "Alt+w", "Rotate selected objects clockwise", NULL,
     CMDID_ROTATECLOCKWISE },
   { N_("RotateCounter"), "Alt+c", "Rotate selected objects counter-clockwise",
     NULL, CMDID_ROTATECOUNTER },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetTextRotation..."), NULL, "Set text rotation in degrees", NULL,
     CMDID_SETTEXTROTATION },
   { N_("SetRotationIncrement..."), NULL, "Set rotation increment in degrees",
     NULL, CMDID_SETROTATIONINCREMENT },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo flipRotateMenuInfo={ TGMUTYPE_TEXT, flipRotateMenuItemInfo,
                                TgCreateMenuFromMenuInfo };

static TgMenuItemInfo arrangeMenuItemInfo[] = {
   { N_("Front"), "Cntrl+f", "Bring selected objects to the front", NULL,
     CMDID_FRONT },
   { N_("Back"), "Cntrl+b", "Send selected objects to the back", NULL,
     CMDID_BACK },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Group"), "Cntrl+g", "Group selected objects into a compound object",
     NULL, CMDID_GROUP },
   { N_("UnGroup"), "Cntrl+u",
     "Break up selected grouped objects into its components", NULL,
     CMDID_UNGROUP },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Lock"), "Alt+<", "Lock the positions of selected objects", NULL,
     CMDID_LOCK },
   { N_("UnLock"), "Alt+>", "UnLock the positions of selected objects", NULL,
     CMDID_UNLOCK },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("HoriAlign"), TGMUITEM_PINNABLESUBMENU,
     "HoriAlign submenu >>>", &horiAlignMenuInfo, MENU_HORIALIGN },
   { N_("VertAlign"), TGMUITEM_PINNABLESUBMENU,
     "VertAlign submenu >>>", &vertAlignMenuInfo,
     MENU_VERTALIGN },
   { N_("AlignObjs"), "Cntrl+l",
     "Align selected objects with each other using the current alignments",
     NULL, CMDID_ALIGNOBJS },
   { N_("AlignObjsDirect"), TGMUITEM_SUBMENU,
     "Align objects direct submenu >>>", &alignObjsDirectMenuInfo, INVALID },
   { N_("AlignToGrid"), "Cntrl+t",
     "Align selected objects to grid points using the current alignments", NULL,
     CMDID_ALIGNTOGRID },
   { N_("AlignToGridDirect"), TGMUITEM_SUBMENU,
     "Align to grid direct submenu >>>", &alignToGridDirectMenuInfo, INVALID },
   { N_("AlignToPage"), "Alt+&",
     "Align selected objects to the page using the current alignments", NULL,
     CMDID_ALIGNTOPAGE },
   { N_("AlignToPageDirect"), TGMUITEM_SUBMENU,
     "Align to page direct submenu >>>", &alignToPageDirectMenuInfo, INVALID },
   { N_("More Object Alignments"), TGMUITEM_SUBMENU,
     "Align objects submenu >>>", &alignMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SizeObjects"), TGMUITEM_SUBMENU,
     "Size objects submenu >>>", &sizeMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("DistributeObjs"), "Alt+l",
     "Distribute selected objects using the current alignments", NULL,
     CMDID_DISTRIBUTEOBJS },
   { N_("DistributeObjsDirect"), TGMUITEM_SUBMENU,
     "Distribute objects direct submenu >>>", &distrObjsDirectMenuInfo,
     INVALID },
   { N_("LayoutOnArc"), NULL, "Layout objects on an arc", NULL,
     CMDID_LAYOUTONARC },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("AbutHorizontal"), "Alt+_",
     "Squish selected objects together horizontally",
     NULL, CMDID_ABUTHORIZONTAL },
   { N_("AbutVertical"), "Alt+|", "Squish selected objects together vertically",
     NULL, CMDID_ABUTVERTICAL },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Flip/Rotate"), TGMUITEM_SUBMENU,
     "Flip and rotate submenu >>>", &flipRotateMenuInfo, INVALID },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo arrangeMenuInfo={ TGMUTYPE_TEXT, arrangeMenuItemInfo,
                             TgCreateMenuFromMenuInfo };

static TgMenuItemInfo scrollModeMenuItemInfo[] = {
   { N_("Off"), NULL, "Do not update canvas window while scrolling", NULL,
     CMDID_CHANGESCROLLMODE },
   { N_("Jump"), NULL, "Jump update canvas window while scrolling", NULL,
     CMDID_CHANGESCROLLMODE },
   { N_("Smooth"), NULL, "Update canvas window smoothly while scrolling", NULL,
     CMDID_CHANGESCROLLMODE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo scrollModeMenuInfo={ TGMUTYPE_TEXT, scrollModeMenuItemInfo,
                                CreateScrollModeMenu };

static TgMenuItemInfo showHideMenuItemInfo[] = {
   { N_("ShowBit/Pixmap"), NULL,
     "Toggle between hiding and showing detailed bitmap/pixmap objects", NULL,
     CMDID_TOGGLEMAPSHOWN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ShowMeasurement"), NULL,
     "Show cursor measurements", NULL, CMDID_TOGGLESHOWMEASUREMENT },
   { N_("SetMeasureUnit..."), NULL, "Set the unit of measurements", NULL,
     CMDID_SETMEASUREUNIT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ShowMenubar"), NULL, "Show menubar window", NULL,
     CMDID_TOGGLESHOWMENUBAR },
   { N_("ShowStatus"), NULL, "Show status window", NULL,
     CMDID_TOGGLESHOWSTATUS },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo showHideMenuInfo={ TGMUTYPE_TEXT, showHideMenuItemInfo,
                                     CreateShowHideMenu };

static TgMenuItemInfo slideShowMenuItemInfo[] = {
   { N_("SlideShow"), "Alt+Cntrl+8", "Toggle SlideShow mode", NULL,
     CMDID_TOGGLESLIDESHOW },
   { N_("SetSlideShowBorderColor..."), NULL,
     "Set the color of the slideshow border", NULL,
     CMDID_SETSLIDESHOWBRDRCOLOR },
   { N_("SetSlideShowWindowOffsets..."), NULL,
     "Set the offsets of the slideshow window", NULL,
     CMDID_SETSLIDESHOWWINOFFSETS },
   { N_("VisibleGridInSlideShow"), NULL,
     "Toggle visible grid in slideshow mode", NULL,
     CMDID_VISIBLEGRIDINSLIDESHOW },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo slideShowMenuInfo={ TGMUTYPE_TEXT, slideShowMenuItemInfo,
                                      CreateSlideShowMenu };

static TgMenuItemInfo zoomMenuItemInfo[] = {
   { N_("ZoomIn"), "Alt+z", "Zoom in", NULL, CMDID_ZOOMIN },
   { N_("ZoomOut"), "Alt+o", "Zoom out", NULL, CMDID_ZOOMOUT },
   { N_("DefaultZoom"), "Alt+:", "No zoom", NULL, CMDID_DEFAULTZOOM },
   { N_("ZoomWayOut"), "Alt+`", "Zoom way out to see the whole drawing", NULL,
     CMDID_ZOOMWAYOUT },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo zoomMenuInfo={ TGMUTYPE_TEXT, zoomMenuItemInfo,
                                 TgCreateMenuFromMenuInfo };

static TgMenuItemInfo layoutMenuItemInfo[] = {
   { N_("+Grid"), "Alt+i", "Increment grid size", NULL, CMDID_INCGRID },
   { N_("-Grid"), "Alt+d", "Decrement grid size", NULL, CMDID_DECGRID },
   { N_("VisibleGrid"), "Alt+g", "Hide/show grid lines", NULL,
     CMDID_TOGGLEGRID },
   { N_("SnapToGrid"), "Alt+Cntrl+g",
     "Toggle the snapping to grid points effect", NULL, CMDID_TOGGLESNAP },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SaveOrigin"), NULL,
     "Save current location as the 'origin' of the drawing",
     NULL, CMDID_SAVEORIGIN },
   { N_("ScrollToOrig"), "Alt+^", "Scroll to the 'origin' of the drawing", NULL,
     CMDID_SCROLLTOORIG },
   { N_("ScrollMode"), TGMUITEM_SUBMENU,
     "ScrollMode submenu >>>", &scrollModeMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Zoom"), TGMUITEM_SUBMENU, "Zoom submenu >>>", &zoomMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Portrait"), NULL, "Portrait page style", NULL, CMDID_PORTRAITMODE },
   { N_("Landscape"), NULL, "Lanscape page style", NULL,
     CMDID_LANDSCAPEMODE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("StackedPage"), TGMUITEM_PINNABLESUBMENU,
     "StackedPage submenu >>>", &stackedPageMenuInfo, MENU_STACKEDPAGE },
   { N_("TiledPage"), TGMUITEM_PINNABLESUBMENU,
     "TiledPage submenu >>>", &tiledPageMenuInfo, MENU_TILEDPAGE },
   { N_("PageLayout"), TGMUITEM_PINNABLESUBMENU,
     "PageLayout submenu >>>", &pageLayoutMenuInfo, MENU_PAGELAYOUT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Print/ExportFormat"), TGMUITEM_SUBMENU,
     "Select print/export format submenu >>>", &exportFormatMenuInfo, INVALID },
   { N_("Print/ExportInColor"), "Alt+Cntrl+k",
     "Toggle between color and black & white printing modes", NULL,
     CMDID_TOGGLEBWCOLORPS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetReduction..."), "Alt+%",
     "Specify reduction/magnification of the whole drawing", NULL,
     CMDID_SETREDUCTION },
   { N_("MetricGrid"), "Alt+Cntrl+-",
     "Toggle between English and Metric grid systems", NULL,
     CMDID_TOGGLEGRIDSYSTEM },
   { N_("OneMotionSelMove"), NULL,
     "Toggle between click-select-move and click-select-click-move modes", NULL,
     CMDID_TOGGLEONEMOTIONSELMOVE },
   { N_("UseGrayScale"), NULL,
     "Toggle between using gray scales to tile patterns to speed up printing",
     NULL, CMDID_TOGGLEUSEGRAYSCALE },
   { N_("UseColorLayers"), NULL, "Toggle color layers", NULL,
     CMDID_TOGGLECOLORLAYERS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Show/Hide"), TGMUITEM_SUBMENU,
     "Show/Hide submenu >>>", &showHideMenuInfo, INVALID },
   { N_("SlideShow"), TGMUITEM_SUBMENU,
     "SlideShow submenu >>>", &slideShowMenuInfo, INVALID },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo layoutMenuInfo={ TGMUTYPE_TEXT, layoutMenuItemInfo,
                            CreateLayoutMenu };

static TgMenuItemInfo propertiesMenuItemInfo[] = {
   { N_("CopyProperties..."), NULL,
     "Copy properties (e.g., color, fill, etc.) into the cut buffer", NULL,
     CMDID_COPYPROPERTIES },
   { N_("PasteProperties..."), NULL,
     "Paste properties (e.g., color, fill, etc.) from the cut buffer", NULL,
     CMDID_PASTEPROPERTIES },
   { N_("SaveProperties..."), NULL,
     "Save and name a property (e.g., color, fill, etc.) set", NULL,
     CMDID_SAVEPROPERTIES },
   { N_("RestoreProperties..."), NULL,
     "Restore a named property (e.g., color, fill, etc.) set", NULL,
     CMDID_RESTOREPROPERTIES },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("UpdateObjects"), "Alt+0",
     "Update selected objects to have current drawing properties", NULL,
     CMDID_UPDATE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetSelLineWidth..."), NULL,
     "Change the line width (and arrow width/height) of selected objects", NULL,
     CMDID_SETSELLINEWIDTH },
   { N_("SetSelFontSize..."), NULL,
     "Change the font size of selected objects", NULL, CMDID_SETSELFONTSIZE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("AddColor..."), NULL, "Add colors to the current palette", NULL,
     CMDID_ADDCOLOR },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SetEditTextSize..."), NULL,
     "Set the text size to be used in editing existing text objects", NULL,
     CMDID_SETEDITTEXTSIZE },
   { N_("SetAltEditTextBgColor..."), NULL,
     "Set the background color to be used in editing existing text objects",
     NULL, CMDID_SETALTEDITTEXTBGCOLOR },
   { N_("SetAltEditTextHighlightColor..."), NULL,
     "Set the highlight color to be used in editing existing text objects",
     NULL, CMDID_SETALTEDITTEXTHIGHLIGHTCOLOR },
   { N_("UseAltEditTextBgColor"), NULL,
     "Toggles the background color to be used in editing existing text objects",
     NULL, CMDID_TOGGLEALTEDITTEXTBGCOLOR },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Font"), TGMUITEM_PINNABLESUBMENU,
     "Font submenu >>>", &fontMenuInfo, MENU_FONT },
   { N_("TextStyle"), TGMUITEM_PINNABLESUBMENU,
     "TextStyle submenu >>>", &fontStyleMenuInfo, MENU_STYLE },
   { N_("TextSize"), TGMUITEM_PINNABLESUBMENU,
     "TextSize submenu >>>", &fontSizeMenuInfo, MENU_SIZE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("LineDash"), TGMUITEM_PINNABLESUBMENU,
     "LineDash submenu >>>", &lineDashMenuInfo, MENU_LINEDASH },
   { N_("LineStyle"), TGMUITEM_PINNABLESUBMENU,
     "LineStyle submenu >>>", &lineStyleMenuInfo, MENU_LINESTYLE },
   { N_("LineType"), TGMUITEM_PINNABLESUBMENU,
     "LineType submenu >>>", &lineTypeMenuInfo, MENU_LINETYPE },
   { N_("LineWidth"), TGMUITEM_PINNABLESUBMENU,
     "LineWidth submenu >>>", &lineWidthMenuInfo, MENU_LINEWIDTH },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Fill"), TGMUITEM_PINNABLESUBMENU,
     "Fill submenu >>>", &fillMenuInfo, MENU_FILL },
   { N_("Pen"), TGMUITEM_PINNABLESUBMENU,
     "Pen submenu >>>", &penMenuInfo, MENU_PEN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("MoveMode"), TGMUITEM_PINNABLESUBMENU,
     "MoveMode submenu >>>", &moveModeMenuInfo, MENU_MOVEMODE },
   { N_("StretchText"), TGMUITEM_PINNABLESUBMENU,
     "StretchText submenu >>>", &stretchableTextModeMenuInfo,
     MENU_STRETCHTEXT },
   { N_("TransPattern"), TGMUITEM_PINNABLESUBMENU,
     "TransPattern submenu >>>", &transPatModeMenuInfo, MENU_TRANSPAT },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo propertiesMenuInfo={ TGMUTYPE_TEXT, propertiesMenuItemInfo,
                                CreatePropertiesMenu };

static TgMenuItemInfo navigateMenuItemInfo[] = {
   { N_("GoBack"), NULL, "Go back one file", NULL, CMDID_GOBACK },
   { N_("GoForward"), NULL, "Go forward one file", NULL, CMDID_GOFORWARD },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("RefreshCurrent"), NULL, "Reload the current file", NULL,
     CMDID_REFRESHCURRENT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("HotList..."), NULL, "Navigate using the hot-list", NULL,
     CMDID_HOTLIST },
   { N_("AddCurrentToHotList"), NULL, "Add the current file to the hot-list",
     NULL, CMDID_ADDCURRENTTOHOTLIST },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("SessionHistory..."), NULL, "Go to a file visited during this session",
     NULL, CMDID_SESSIONHISTORY },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("HyperSpace"), NULL, "Toggle HyperSpace mode", NULL,
     CMDID_TOGGLEHYPERSPACE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo navigateMenuInfo={ TGMUTYPE_TEXT, navigateMenuItemInfo,
                              CreateNavigateMenu };

static TgMenuItemInfo attrMenuItemInfo[] = {
   { N_("AttachAttrs"), "Alt+a",
     "Attach selected text objects as attributes for the non-text object", NULL,
     CMDID_ATTACHATTRS },
   { N_("DetachAttrs"), "Alt+t",
     "Detach all attributes of selected objects", NULL,
     CMDID_DETACHATTRS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("EditAttrs..."), NULL,
     "Edit attributes of a selected object", NULL,
     CMDID_EDITATTRS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ShowAttr"), "Alt+-",
     "Make all attributes of selected objects visible", NULL,
     CMDID_SHOWATTR },
   { N_("ShowAttrName"), "Alt+n",
     "Make all attribute names of selected objects visible", NULL,
     CMDID_SHOWATTRNAME },
   { N_("HideAttr"), "Alt+Cntrl+h",
     "Hide all attributes for selected objects", NULL,
     CMDID_HIDEATTR },
   { N_("HideAttrName"), "Alt+j",
     "Hide all attribute names for selected objects", NULL,
     CMDID_HIDEATTRNAME },
   { N_("ToggleEqAttrShown..."), "Alt+Cntrl+7",
     "Showing/hiding the 'eq=' attibute (for a LaTeX equation object)",
     NULL, CMDID_TOGGLEEQATTRSHOWN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Move/JustfyAttr"), "Alt+m",
     "Move/justify an attribute for a selected object", NULL,
     CMDID_MOVEJUSTIFYATTR },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ImportAttrs..."), NULL,
     "Import attributes of a selected object from a text file", NULL,
     CMDID_IMPORTATTRS },
   { N_("ExportAttrs..."), NULL,
     "Export attributes of a selected object to a text file", NULL,
     CMDID_EXPORTATTRS },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo attrMenuInfo={ TGMUTYPE_TEXT, attrMenuItemInfo,
                                 TgCreateMenuFromMenuInfo };

static TgMenuItemInfo fileAttrMenuItemInfo[] = {
   { N_("AttachFileAttrs"), NULL,
     "Attach selected text objects as file attributes", NULL,
     CMDID_ATTACHFILEATTRS },
   { N_("DetachFileAttrs"), NULL,
     "Detach all file attributes", NULL,
     CMDID_DETACHFILEATTRS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("EditFileAttrs..."), NULL,
     "Edit file attributes", NULL,
     CMDID_EDITFILEATTRS },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo fileAttrMenuInfo={ TGMUTYPE_TEXT, fileAttrMenuItemInfo,
                                 TgCreateMenuFromMenuInfo };

static TgMenuItemInfo animateMenuItemInfo[] = {
   { N_("AnimateSend"), "Alt+e",
     "Animate a little token on a selected poly (not very useful)", NULL,
     CMDID_ANIMATESEND },
   { N_("AnimateFlash"), "Alt+f",
     "Flash a selected poly (not very useful)", NULL,
     CMDID_ANIMATEFLASH },
   { NULL, NULL, NULL, NULL, INVALID }
};
static TgMenuInfo animateMenuInfo={ TGMUTYPE_TEXT, animateMenuItemInfo,
                                    TgCreateMenuFromMenuInfo };

static TgMenuItemInfo specialMenuItemInfo[] = {
   { N_("MakeSymbolic"), "Alt+Cntrl+m",
     "Turn a selected grouped/icon object into a symbol object", NULL,
     CMDID_MAKESYMBOLIC },
   { N_("UnMakeSymbolic"), "Alt+Cntrl+n",
     "Turn a selected symbol object into a grouped object", NULL,
     CMDID_UNMAKESYMBOLIC },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Instantiate..."), "Cntrl+i",
     "Instantiate a building-block object from the current domain", NULL,
     CMDID_INSTANTIATE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("MakeIconic..."), "Alt+Cntrl+i",
     "Turn a selected grouped/symbol object into an icon object", NULL,
     CMDID_MAKEICONIC },
   { N_("UnMakeIconic"), "Alt+Cntrl+j",
     "Turn an icon object into a grouped object", NULL,
     CMDID_UNMAKEICONIC },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Push"), "Alt+Cntrl+y",
     "Push into (edit) the symbol file which defines the selected icon object",
     NULL, CMDID_PUSH },
   { N_("Pop"), "Cntrl+k",
     "Pop back to a high lever (reverse of Push)", NULL,
     CMDID_POP },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Attribute"), TGMUITEM_SUBMENU,
     "Attribute submenu >>>", &attrMenuInfo, INVALID },
   { N_("FileAttribute"), TGMUITEM_SUBMENU,
     "File attribute submenu >>>", &fileAttrMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("UpdateSymbols"), "Alt+Cntrl+u",
     "Refresh selected icon objects from their definition files", NULL,
     CMDID_UPDATESYMBOLS },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Misc"), TGMUITEM_SUBMENU,
     "Misc submenu >>>", &animateMenuInfo, INVALID },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("MergeWithTable..."), NULL,
     "Merge selected object with a table file", NULL,
     CMDID_MERGEWITHTABLE },
   { N_("ExportToTable..."), NULL,
     "Export attributes of selected objects to a table file", NULL,
     CMDID_EXPORTTOTABLE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ExecCmdsFromFile..."), NULL, "Execute internal commands from a file",
     NULL, CMDID_EXECCMDSFROMFILE },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo specialMenuInfo={ TGMUTYPE_TEXT, specialMenuItemInfo,
                             TgCreateMenuFromMenuInfo };

static TgMenuItemInfo helpMenuItemInfo[] = {
   { N_("LatestReleaseInfo"), NULL,
     "Find out what is the version number of the most recently released tgif",
     NULL, CMDID_LATESTRELEASEINFO },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Copyright"), NULL, "Copyright information", NULL, CMDID_COPYRIGHT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("About"), NULL,
     "Version information, where to send bug reports, etc.", NULL,
     CMDID_ABOUT },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo helpMenuInfo={ TGMUTYPE_TEXT, helpMenuItemInfo,
                          TgCreateMenuFromMenuInfo };

/* ------------------ main menu ------------------ */

static TgMenuItemInfo mainMenuItemInfo[] = {
   { N_("File"), TGMUITEM_PINNABLESUBMENU, "File Menu", &fileMenuInfo,
     MENU_FILE },
   { N_("Edit"), TGMUITEM_PINNABLESUBMENU, "Edit Menu", &editMenuInfo,
     MENU_EDIT },
   { N_("Layout"), TGMUITEM_PINNABLESUBMENU,
     "Layout Menu", &layoutMenuInfo, MENU_LAYOUT },
   { N_("Arrange"), TGMUITEM_PINNABLESUBMENU,
     "Arrange Menu", &arrangeMenuInfo, MENU_ARRANGE },
   { N_("Properties"), TGMUITEM_PINNABLESUBMENU,
     "Properties Menu", &propertiesMenuInfo, MENU_PROPERTIES },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("StackedPage"), TGMUITEM_PINNABLESUBMENU,
     "StackedPage Menu", &stackedPageMenuInfo, MENU_STACKEDPAGE },
   { N_("TiledPage"), TGMUITEM_PINNABLESUBMENU,
     "TiledPage Menu", &tiledPageMenuInfo, MENU_TILEDPAGE },
   { N_("PageLayout"), TGMUITEM_PINNABLESUBMENU,
     "PageLayout Menu", &pageLayoutMenuInfo, MENU_PAGELAYOUT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("HoriAlign"), TGMUITEM_PINNABLESUBMENU,
     "HoriAlign Menu", &horiAlignMenuInfo, MENU_HORIALIGN },
   { N_("VertAlign"), TGMUITEM_PINNABLESUBMENU,
     "VertAlign Menu", &vertAlignMenuInfo, MENU_VERTALIGN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Font"), TGMUITEM_PINNABLESUBMENU, "Font Menu", &fontMenuInfo,
     MENU_FONT },
   { N_("TextStyle"), TGMUITEM_PINNABLESUBMENU,
     "TextStyle Menu", &fontStyleMenuInfo, MENU_STYLE },
   { N_("TextSize"), TGMUITEM_PINNABLESUBMENU,
     "TextSize Menu", &fontSizeMenuInfo, MENU_SIZE },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("LineDash"), TGMUITEM_PINNABLESUBMENU,
     "LineDash Menu", &lineDashMenuInfo, MENU_LINEDASH },
   { N_("LineStyle"), TGMUITEM_PINNABLESUBMENU,
     "LineStyle Menu", &lineStyleMenuInfo, MENU_LINESTYLE },
   { N_("LineType"), TGMUITEM_PINNABLESUBMENU,
     "LineType Menu", &lineTypeMenuInfo, MENU_LINETYPE },
   { N_("LineWidth"), TGMUITEM_PINNABLESUBMENU,
     "LineWidth Menu", &lineWidthMenuInfo, MENU_LINEWIDTH },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Fill"), TGMUITEM_PINNABLESUBMENU, "Fill Menu", &fillMenuInfo,
     MENU_FILL },
   { N_("Pen"), TGMUITEM_PINNABLESUBMENU, "Pen Menu", &penMenuInfo, MENU_PEN },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("MoveMode"), TGMUITEM_PINNABLESUBMENU,
     "MoveMode Menu", &moveModeMenuInfo, MENU_MOVEMODE },
   { N_("Shape"), TGMUITEM_PINNABLESUBMENU,
     "Shape Menu", &shapeMenuInfo, MENU_SHAPE },
   { N_("StretchText"), TGMUITEM_PINNABLESUBMENU,
     "StretchText Menu", &stretchableTextModeMenuInfo, MENU_STRETCHTEXT },
   { N_("TransPattern"), TGMUITEM_PINNABLESUBMENU,
     "TransPattern Menu", &transPatModeMenuInfo,
         MENU_TRANSPAT },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("ImageProc"), TGMUITEM_PINNABLESUBMENU,
     "ImageProc Menu", &imageProcMenuInfo, MENU_IMAGEPROC },
   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
   { N_("Color"), TGMUITEM_PINNABLESUBMENU,
     "Color Menu", &colorMenuInfo, MENU_COLOR },
   { N_("Navigate"), TGMUITEM_PINNABLESUBMENU,
     "Navigate Menu", &navigateMenuInfo, MENU_NAVIGATE },
   { N_("Special"), TGMUITEM_PINNABLESUBMENU,
     "Special Menu", &specialMenuInfo, MENU_SPECIAL },
#ifdef _TANGRAM_II
   { N_("Tangram-II"), TGMUITEM_PINNABLESUBMENU,
     "Tangram-II Menu", &tangram2MenuInfo, MENU_TANGRAM2 },
#endif /* _TANGRAM_II */
   { N_("Help"), TGMUITEM_PINNABLESUBMENU, "Help Menu", &helpMenuInfo,
     MENU_HELP },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo mainMenuInfo={ TGMUTYPE_TEXT, mainMenuItemInfo, CreateMainMenu };

/* ------------------ menubar ------------------ */

/*
 * Don't put any separators here!
 */

static TgMenuItemInfo minimalMenubarMenuItemInfo[] = {
   { N_("File"), TGMUITEM_PINNABLESUBMENU, "File Menu", &fileMenuInfo,
     MENU_FILE },
   { N_("Edit"), TGMUITEM_PINNABLESUBMENU, "Edit Menu", &editMenuInfo,
     MENU_EDIT },
   { N_("Layout"), TGMUITEM_PINNABLESUBMENU,
     "Layout Menu", &layoutMenuInfo, MENU_LAYOUT },
   { N_("Arrange"), TGMUITEM_PINNABLESUBMENU,
     "Arrange Menu", &arrangeMenuInfo, MENU_ARRANGE },
   { N_("Properties"), TGMUITEM_PINNABLESUBMENU,
     "Properties Menu", &propertiesMenuInfo, MENU_PROPERTIES },
   { N_("Color"), TGMUITEM_PINNABLESUBMENU,
     "Color Menu", &colorMenuInfo, MENU_COLOR },
   { N_("Navigate"), TGMUITEM_PINNABLESUBMENU,
     "Navigate Menu", &navigateMenuInfo, MENU_NAVIGATE },
   { N_("Special"), TGMUITEM_PINNABLESUBMENU,
     "Special Menu", &specialMenuInfo, MENU_SPECIAL },
#ifdef _TANGRAM_II
   { N_("Tangram-II"), TGMUITEM_PINNABLESUBMENU,
     "Tangram-II Menu", &tangram2MenuInfo, MENU_TANGRAM2 },
#endif /* _TANGRAM_II */
   { N_("Help"), TGMUITEM_PINNABLESUBMENU, "Help Menu", &helpMenuInfo,
     MENU_HELP },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo minimalMenubarMenuInfo={ TGMUTYPE_TEXT, minimalMenubarMenuItemInfo,
                                    NULL };

static TgMenuItemInfo maximalMenubarMenuItemInfo[] = {
   { N_("File"), TGMUITEM_PINNABLESUBMENU, "File Menu", &fileMenuInfo,
     MENU_FILE },
   { N_("Edit"), TGMUITEM_PINNABLESUBMENU, "Edit Menu", &editMenuInfo,
     MENU_EDIT },
   { N_("Layout"), TGMUITEM_PINNABLESUBMENU,
     "Layout Menu", &layoutMenuInfo, MENU_LAYOUT },
   { N_("Arrange"), TGMUITEM_PINNABLESUBMENU,
     "Arrange Menu", &arrangeMenuInfo, MENU_ARRANGE },
   { N_("Properties"), TGMUITEM_PINNABLESUBMENU,
     "Properties Menu", &propertiesMenuInfo, MENU_PROPERTIES },
   { N_("MoveMode"), TGMUITEM_PINNABLESUBMENU,
     "MoveMode Menu", &moveModeMenuInfo, MENU_MOVEMODE },
   { N_("Page"), TGMUITEM_PINNABLESUBMENU,
     "Page Menu", &pageMenuInfo, MENU_PAGE },
   { N_("PageLayout"), TGMUITEM_PINNABLESUBMENU,
     "PageLayout Menu", &pageLayoutMenuInfo, MENU_PAGELAYOUT },
   { N_("HoriAlign"), TGMUITEM_PINNABLESUBMENU,
     "HoriAlign Menu", &horiAlignMenuInfo, MENU_HORIALIGN },
   { N_("VertAlign"), TGMUITEM_PINNABLESUBMENU,
     "VertAlign Menu", &vertAlignMenuInfo, MENU_VERTALIGN },
   { N_("Font"), TGMUITEM_PINNABLESUBMENU, "Font Menu", &fontMenuInfo,
     MENU_FONT },
   { N_("TextStyle"), TGMUITEM_PINNABLESUBMENU,
     "TextStyle Menu", &fontStyleMenuInfo, MENU_STYLE },
   { N_("TextSize"), TGMUITEM_PINNABLESUBMENU,
     "TextSize Menu", &fontSizeMenuInfo, MENU_SIZE },
   { N_("Shape"), TGMUITEM_PINNABLESUBMENU,
     "Shape Menu", &shapeMenuInfo, MENU_SHAPE },
   { N_("StretchText"), TGMUITEM_PINNABLESUBMENU,
     "StretchText Menu", &stretchableTextModeMenuInfo, MENU_STRETCHTEXT },
   { N_("LineDash"), TGMUITEM_PINNABLESUBMENU,
     "LineDash Menu", &lineDashMenuInfo, MENU_LINEDASH },
   { N_("LineStyle"), TGMUITEM_PINNABLESUBMENU,
     "LineStyle Menu", &lineStyleMenuInfo, MENU_LINESTYLE },
   { N_("LineType"), TGMUITEM_PINNABLESUBMENU,
     "LineType Menu", &lineTypeMenuInfo, MENU_LINETYPE },
   { N_("LineWidth"), TGMUITEM_PINNABLESUBMENU,
     "LineWidth Menu", &lineWidthMenuInfo, MENU_LINEWIDTH },
   { N_("Fill"), TGMUITEM_PINNABLESUBMENU, "Fill Menu", &fillMenuInfo,
     MENU_FILL },
   { N_("Pen"), TGMUITEM_PINNABLESUBMENU, "Pen Menu", &penMenuInfo, MENU_PEN },
   { N_("TransPattern"), TGMUITEM_PINNABLESUBMENU,
     "TransPattern Menu", &transPatModeMenuInfo, MENU_TRANSPAT },
   { N_("Color"), TGMUITEM_PINNABLESUBMENU,
     "Color Menu", &colorMenuInfo, MENU_COLOR },
   { N_("ImageProc"), TGMUITEM_PINNABLESUBMENU,
     "ImageProc Menu", &imageProcMenuInfo, MENU_IMAGEPROC },
   { N_("Navigate"), TGMUITEM_PINNABLESUBMENU,
     "Navigate Menu", &navigateMenuInfo, MENU_NAVIGATE },
   { N_("Special"), TGMUITEM_PINNABLESUBMENU,
     "Special Menu", &specialMenuInfo, MENU_SPECIAL },
#ifdef _TANGRAM_II
   { N_("Tangram-II"), TGMUITEM_PINNABLESUBMENU,
     "Tangram-II Menu", &tangram2MenuInfo, MENU_TANGRAM2 },
#endif /* _TANGRAM_II */
   { N_("Help"), TGMUITEM_PINNABLESUBMENU, "Help Menu", &helpMenuInfo,
     MENU_HELP },
   { NULL, NULL, NULL, NULL, INVALID }
};
TgMenuInfo maximalMenubarMenuInfo={ TGMUTYPE_TEXT, maximalMenubarMenuItemInfo,
                                    NULL };

